package explorviz.live_trace_processing.filter.reduction;

import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import explorviz.live_trace_processing.filter.reduction.summarization.AssociativityFilter;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.event.normal.AfterOperationEventRecord;
import explorviz.live_trace_processing.record.event.normal.BeforeOperationEventRecord;
import explorviz.live_trace_processing.record.trace.HostApplicationMetaDataRecord;
import explorviz.live_trace_processing.record.trace.RuntimeStatisticInformation;
import explorviz.live_trace_processing.record.trace.Trace;

public class AssociativityFilterTest {

	private Trace testTrace;

	@Before
	public void init() {
		final List<AbstractEventRecord> traceEvents = new ArrayList<AbstractEventRecord>();

		final BeforeOperationEventRecord beforeEvent = new BeforeOperationEventRecord(1L, 1L, 0, 0,
				"test.package.Clazz.method1()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent.setRuntimeStatisticInformation(new RuntimeStatisticInformation(11L));
		traceEvents.add(beforeEvent);

		final BeforeOperationEventRecord beforeEvent1 = new BeforeOperationEventRecord(2L, 1L, 1,
				0, "test.package.Clazz.method2()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent1.setRuntimeStatisticInformation(new RuntimeStatisticInformation(2L));
		traceEvents.add(beforeEvent1);

		final AfterOperationEventRecord afterEvent1 = new AfterOperationEventRecord(4L, 1L, 2,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent1);

		final BeforeOperationEventRecord beforeEvent2 = new BeforeOperationEventRecord(5L, 1L, 3,
				0, "test.package.OtherClazz.method3()", "ClazzF",
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		beforeEvent2.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent2);

		final AfterOperationEventRecord afterEvent2 = new AfterOperationEventRecord(6L, 1L, 4,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent2);

		final BeforeOperationEventRecord beforeEvent3 = new BeforeOperationEventRecord(7L, 1L, 5,
				0, "test.package.Clazz.method2()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent3.setRuntimeStatisticInformation(new RuntimeStatisticInformation(2L));
		traceEvents.add(beforeEvent3);

		final AfterOperationEventRecord afterEvent3 = new AfterOperationEventRecord(9L, 1L, 6,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent3);

		final BeforeOperationEventRecord beforeEvent4 = new BeforeOperationEventRecord(10L, 1L, 7,
				0, "test.package.Clazz.method2()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent4.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent4);

		final AfterOperationEventRecord afterEvent4 = new AfterOperationEventRecord(11L, 1L, 8,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent4);

		final AfterOperationEventRecord afterEvent = new AfterOperationEventRecord(12L, 1L, 9,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent);

		testTrace = new Trace(traceEvents, true);
	}

	@Test
	public void testReduction() {
		final List<AbstractEventRecord> traceEvents = new ArrayList<AbstractEventRecord>();

		final BeforeOperationEventRecord beforeEvent = new BeforeOperationEventRecord(1L, 1L, 0, 0,
				"test.package.Clazz.method1()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent.setRuntimeStatisticInformation(new RuntimeStatisticInformation(11L));
		traceEvents.add(beforeEvent);

		final BeforeOperationEventRecord beforeEvent2 = new BeforeOperationEventRecord(5L, 1L, 3,
				0, "test.package.OtherClazz.method3()", "ClazzF",
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		beforeEvent2.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent2);

		final AfterOperationEventRecord afterEvent2 = new AfterOperationEventRecord(6L, 1L, 4,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent2);

		final AfterOperationEventRecord afterEvent = new AfterOperationEventRecord(12L, 1L, 9,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent);

		final Trace compareTrace = new Trace(traceEvents, true);
		final AssociativityFilter assoziationFilter = new AssociativityFilter(null);
		final Trace reducedTrace = assoziationFilter.testReduction(testTrace);
		Assert.assertTrue(compareTrace.compareTo(reducedTrace) == 0);

	}

}
