package explorviz.live_trace_processing.filter.reduction;

import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.event.normal.AfterOperationEventRecord;
import explorviz.live_trace_processing.record.event.normal.BeforeOperationEventRecord;
import explorviz.live_trace_processing.record.trace.HostApplicationMetaDataRecord;
import explorviz.live_trace_processing.record.trace.RuntimeStatisticInformation;

public class PatternSummarizationTest {

	@Before
	public void init() {
		final List<AbstractEventRecord> traceEvents = new ArrayList<AbstractEventRecord>();

		final BeforeOperationEventRecord beforeEvent = new BeforeOperationEventRecord(1L, 1L, 0, 0,
				"test.package.Clazz.A()", "ClazzA", new HostApplicationMetaDataRecord("testSystem",
						"testIp", "host", "application"));
		beforeEvent.setRuntimeStatisticInformation(new RuntimeStatisticInformation(31L));
		traceEvents.add(beforeEvent);

		final BeforeOperationEventRecord beforeEvent1 = new BeforeOperationEventRecord(2L, 1L, 1,
				0, "test.package.Clazz.B()", "ClazzB", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent1.setRuntimeStatisticInformation(new RuntimeStatisticInformation(2L));
		traceEvents.add(beforeEvent1);

		final AfterOperationEventRecord afterEvent1 = new AfterOperationEventRecord(4L, 1L, 2,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent1);

		final BeforeOperationEventRecord beforeEvent2 = new BeforeOperationEventRecord(5L, 1L, 3,
				0, "test.package.Clazz.C()", "ClazzC", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent2.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent2);

		final AfterOperationEventRecord afterEvent2 = new AfterOperationEventRecord(6L, 1L, 4,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent2);

		final BeforeOperationEventRecord beforeEvent3 = new BeforeOperationEventRecord(7L, 1L, 5,
				0, "test.package.Clazz.D()", "ClazzD", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent3.setRuntimeStatisticInformation(new RuntimeStatisticInformation(9L));
		traceEvents.add(beforeEvent3);

		final BeforeOperationEventRecord beforeEvent4 = new BeforeOperationEventRecord(8L, 1L, 6,
				0, "test.package.Clazz.E()", "ClazzE", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent4.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent4);

		final AfterOperationEventRecord afterEvent4 = new AfterOperationEventRecord(9L, 1L, 7,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent4);

		final BeforeOperationEventRecord beforeEvent5 = new BeforeOperationEventRecord(10L, 1L, 8,
				0, "test.package.Clazz.F()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent5.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent5);

		final AfterOperationEventRecord afterEvent5 = new AfterOperationEventRecord(11L, 1L, 9,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent5);

		final BeforeOperationEventRecord beforeEvent6 = new BeforeOperationEventRecord(12L, 1L, 10,
				0, "test.package.Clazz.E()", "ClazzE", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent6.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent6);

		final AfterOperationEventRecord afterEvent6 = new AfterOperationEventRecord(13L, 1L, 11,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent6);

		final BeforeOperationEventRecord beforeEvent7 = new BeforeOperationEventRecord(14L, 1L, 12,
				0, "test.package.Clazz.F()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent7.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent7);

		final AfterOperationEventRecord afterEvent7 = new AfterOperationEventRecord(15L, 1L, 13,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent7);

		final AfterOperationEventRecord afterEvent3 = new AfterOperationEventRecord(16L, 1L, 14,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent3);

		final BeforeOperationEventRecord beforeEvent8 = new BeforeOperationEventRecord(17L, 1L, 15,
				0, "test.package.Clazz.B()", "ClazzB", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent8.setRuntimeStatisticInformation(new RuntimeStatisticInformation(2L));
		traceEvents.add(beforeEvent8);

		final AfterOperationEventRecord afterEvent8 = new AfterOperationEventRecord(19L, 1L, 16,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent8);

		final BeforeOperationEventRecord beforeEvent9 = new BeforeOperationEventRecord(20L, 1L, 17,
				0, "test.package.Clazz.C()", "ClazzC", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent9.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent9);

		final AfterOperationEventRecord afterEvent9 = new AfterOperationEventRecord(21L, 1L, 18,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent9);

		final BeforeOperationEventRecord beforeEvent10 = new BeforeOperationEventRecord(22L, 1L,
				19, 0, "test.package.Clazz.D()", "ClazzD", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent10.setRuntimeStatisticInformation(new RuntimeStatisticInformation(9L));
		traceEvents.add(beforeEvent10);

		final BeforeOperationEventRecord beforeEvent11 = new BeforeOperationEventRecord(23L, 1L,
				20, 0, "test.package.Clazz.E()", "ClazzE", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent11.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent11);

		final AfterOperationEventRecord afterEvent11 = new AfterOperationEventRecord(24L, 1L, 21,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent11);

		final BeforeOperationEventRecord beforeEvent12 = new BeforeOperationEventRecord(25L, 1L,
				22, 0, "test.package.Clazz.F()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent12.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent12);

		final AfterOperationEventRecord afterEvent12 = new AfterOperationEventRecord(26L, 1L, 23,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent12);

		final BeforeOperationEventRecord beforeEvent13 = new BeforeOperationEventRecord(27L, 1L,
				24, 0, "test.package.Clazz.E()", "ClazzE", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent13.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent13);

		final AfterOperationEventRecord afterEvent13 = new AfterOperationEventRecord(28L, 1L, 25,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent13);

		final BeforeOperationEventRecord beforeEvent14 = new BeforeOperationEventRecord(29L, 1L,
				26, 0, "test.package.Clazz.F()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent14.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1L));
		traceEvents.add(beforeEvent14);

		final AfterOperationEventRecord afterEvent14 = new AfterOperationEventRecord(30L, 1L, 27,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent14);

		final AfterOperationEventRecord afterEvent10 = new AfterOperationEventRecord(31L, 1L, 28,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent10);

		final AfterOperationEventRecord afterEvent = new AfterOperationEventRecord(32L, 1L, 29,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent);

		// trace = new Trace(traceEvents, true);
	}

	@Test
	public void testReduction() {
		final List<AbstractEventRecord> traceEvents = new ArrayList<AbstractEventRecord>();

		final BeforeOperationEventRecord beforeEvent = new BeforeOperationEventRecord(1L, 1L, 0, 0,
				"test.package.Clazz.A()", "ClazzA", new HostApplicationMetaDataRecord("testSystem",
						"testIp", "host", "application"));
		beforeEvent.setRuntimeStatisticInformation(new RuntimeStatisticInformation(31L));
		traceEvents.add(beforeEvent);

		final BeforeOperationEventRecord beforeEvent1 = new BeforeOperationEventRecord(2L, 1L, 1,
				0, "test.package.Clazz.B()", "ClazzB", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent1.setRuntimeStatisticInformation(new RuntimeStatisticInformation(2, 4, 8));
		traceEvents.add(beforeEvent1);

		final AfterOperationEventRecord afterEvent1 = new AfterOperationEventRecord(4L, 1L, 2,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent1);

		final BeforeOperationEventRecord beforeEvent2 = new BeforeOperationEventRecord(5L, 1L, 3,
				0, "test.package.Clazz.C()", "ClazzC", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent2.setRuntimeStatisticInformation(new RuntimeStatisticInformation(2, 2, 2));
		traceEvents.add(beforeEvent2);

		final AfterOperationEventRecord afterEvent2 = new AfterOperationEventRecord(6L, 1L, 4,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent2);

		final BeforeOperationEventRecord beforeEvent3 = new BeforeOperationEventRecord(7L, 1L, 5,
				0, "test.package.Clazz.D()", "ClazzD", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent3.setRuntimeStatisticInformation(new RuntimeStatisticInformation(2, 18, 162));
		traceEvents.add(beforeEvent3);

		final BeforeOperationEventRecord beforeEvent4 = new BeforeOperationEventRecord(8L, 1L, 6,
				0, "test.package.Clazz.E()", "ClazzE", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent4.setRuntimeStatisticInformation(new RuntimeStatisticInformation(4, 4, 4));
		traceEvents.add(beforeEvent4);

		final AfterOperationEventRecord afterEvent4 = new AfterOperationEventRecord(9L, 1L, 7,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent4);

		final BeforeOperationEventRecord beforeEvent5 = new BeforeOperationEventRecord(10L, 1L, 8,
				0, "test.package.Clazz.F()", "ClazzF", new HostApplicationMetaDataRecord(
						"testSystem", "testIp", "host", "application"));
		beforeEvent5.setRuntimeStatisticInformation(new RuntimeStatisticInformation(4, 4, 4));
		traceEvents.add(beforeEvent5);

		final AfterOperationEventRecord afterEvent5 = new AfterOperationEventRecord(11L, 1L, 9,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent5);

		final AfterOperationEventRecord afterEvent3 = new AfterOperationEventRecord(16L, 1L, 14,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent3);

		final AfterOperationEventRecord afterEvent = new AfterOperationEventRecord(32L, 1L, 29,
				new HostApplicationMetaDataRecord("testSystem", "testIp", "host", "application"));
		traceEvents.add(afterEvent);

		// final Trace compareTrace = new Trace(traceEvents, true);
		// final PatternSummarizationFilter patternSummarizationFilter = new
		// PatternSummarizationFilter(
		// null, PatternSummarizationFilter.DEFAULT_DISTANCE,
		// PatternSummarizationFilter.DEFAULT_NESTINGLEVEL);
		// final Trace reducedTrace =
		// patternSummarizationFilter.testReduction(trace);
		// Assert.assertTrue(compareTrace.compareTo(reducedTrace) == 0);

	}

}
